/*
 Copyright (C) 2011 J. Coliz <maniacbug@ymail.com>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.
 */

/**
 * Channel scanner
 *
 * Example to detect interference on the various channels available.
 * This is a good diagnostic tool to check whether you're picking a
 * good channel for your application.
 *
 * Inspired by cpixip.
 * See http://arduino.cc/forum/index.php/topic,54795.0.html
 */

#include <SPI.h>
#include "nRF24L01.h"
#include "RF24.h"
#include "printf.h"
//
// Hardware configuration
//

// Set up nRF24L01 radio on SPI bus plus CE and CSN
// CE  - Chip Enable
// CSN - Chip Select

#define CE_PIN  9
#define CSN_PIN 10

RF24 radio(CE_PIN,CSN_PIN);

//
// Channel info
//

const uint8_t num_channels = 126;
uint8_t values[num_channels];
uint8_t weak[num_channels];
uint64_t add;

/*
// define WR address straight and reversed
uint64_t address =0x1319406101;
uint64_t address2=0x6190746601;
int width = 5;
*/

// define preamble as address to sniff any traffic
uint64_t address2=0x0055;
uint64_t address=0x00aa;
int width = 2;

// define channel as i
//int i = 0x29;
int i = 3;

//void* data;
byte data[32];
unsigned long timestamp;

void setup(void) {
  Serial.begin(115200);
  printf_begin();
  printf("\n\rRF24/examples/scanner/\n\r");

  radio.begin();
  radio.setChannel(i);
  radio.setAddressWidth(width);
  radio.setPayloadSize(32);
  radio.setDataRate(RF24_250KBPS); //RF24_250KBPS for 250kbs, RF24_1MBPS for 1Mbps, or RF24_2MBPS
  radio.setAutoAck(false);
  radio.disableCRC();
  //radio.setCRCLength(RF24_CRC_8); //  RF24_CRC_8 for 8-bit or RF24_CRC_16 for 16-bit 

  radio.openReadingPipe(0, address);
  radio.openReadingPipe(1, address2);
  radio.printDetails();

  radio.startListening();
  radio.stopListening();

  int d = 0;
}

//
// Loop
//

const int num_reps = 1;
uint8_t pipeNum;
int car=2;
bool goodSignal = false;
int arr[]={0x7f,0x7e,0x50,0x4e,0x4c,0x47,0x43,0x40,0x3d,0x3b,0x3a,0x39,0x38,0x37,0x36,0x35,0x33,0x32,0x31,0x30,0x2f,0x2d,0x2b,0x29,0x1b,0x1a,0x16,0x02};

void loop(void) {
   memset(values,0,sizeof(values));
   memset(weak,0,sizeof(weak));
   
   radio.openReadingPipe(0, address);
   radio.openReadingPipe(1, address2);
   radio.startListening();
   while (true) {
      while (true) {
        if (radio.available(&pipeNum)) break;
        delay(2);
      }
      
      //delay(80); //0.1 second
       
      // Did we get a carrier?
      /*
      if ( radio.testCarrier() ){
         goodSignal = radio.testRPD();
         goodSignal ? ++values[i] : ++weak[i]; 
         car = 1;
      } else {
         goodSignal = false;
         car = 0;
      }
      */
      if (true) {
         (pipeNum == 0) ? add=address : add=address2;
         memset(data,0x00,sizeof(data));
         radio.read(&data,sizeof(data));
         if (car=1){
           //check if byte 4,5,6 defines a valid pl-length
           byte pls[2+1];
           bool pls_ok[2+1];
           int pid[2+1];
           int noack[2+1];
           int g=2;
           //for (int g = 0; g < 2+1; g++) { 
              pls[g] = data[g+3]>>2;
              pls_ok[g] = ((pls[g] >=0) && (pls[g]<= 32));
              pid[g] = data[g+3] & 3;
              noack[g]=data[g+4]>>7;
           //}
//           for (int d = 0; d < sizeof(data); d++) { printf("%02x ", data[d]); }
//           timestamp = millis();
//           printf("- ch:0x%02x size:%d carr:%d time:%ld good:%d 3:%i/%i/%i 4:%i/%i/%i 5:%i/%i/%i p:%d addr:0x%04x",
//                          i, sizeof(data), car, timestamp, goodSignal, pls[0],pid[0],noack[0], pls[1],pid[1],noack[1], pls[2],pid[2],noack[2], pipeNum, add);
//           printf("\n\r");
           //repeat this line, move payload 1 bit to see payload after no_ack flag
           if (pls_ok[2]) {
             for (int d = 0; d <= 4; d++) { printf("%02x ", data[d]); } 
             printf("(%2i/%i/%i) ", pls[2],pid[2],noack[2]);
             for (int d = 6; d < sizeof(data)-1; d++) { 
                byte shifted = data[d]<<1;
                if (data[d+1]>>7) { bitSet(shifted,0); }
                printf("%02x", shifted); 
                if (d==6) printf("("); //destination starts, shoud be same as first 5 bytes
                else if (d==10) printf("|"); // source address starts
                else if (d==14) printf(")"); // command after this
                else if (d==6+pls[2]-1) printf("["); // end of payload
                else if (d==6+pls[2]+1) printf("]"); // end of crc16
                else printf(" ");
             } 
             //printf(" - ch:0x%02x size:%d, carr:%d time:%ld good:%d 5:%i/%i/%i addr:0x%04x shiftbit9", i, sizeof(data), car, timestamp, goodSignal, pls[2],pid[2],noack[2], add);
             printf(" - ch:0x%02x size:%d time:%ld addr:0x%04x shiftbit9", i, sizeof(data), timestamp, add);
             printf("\n\r");
           }
         }
      }
   }
}

// vim:ai:cin:sts=2 sw=2 ft=cpp
